#!/usr/bin/env Rscript

options(stringsAsFactors=FALSE)
library(stringr)
library(abind)

# helper function
LoadToEnvironment <- function(RData, env = new.env()){
  load(RData, env)
  return(env) 
}

args = commandArgs(trailingOnly=TRUE)

if(length(args)==0 | any(sapply(args, function(fname) !grepl(".RData$",fname)) )) {
  print("Error in merge input")
  quit(status=1)
}

output = args[1]

input = args[-1]

env = LoadToEnvironment(input[1])
out = env$out

# if viapredict return NULL, quit (most likely: too few cell lines)
# need to save output for snakemake to continue
if(is.null(out)) {
  print("Is null")
  save(out, file=output)
  quit(status=0)
}


has_preds = 'preds_array' %in% names(out)
has_models = 'models' %in% names(out)
if(has_preds) {
  preds_array = out$preds_array
  data_y = out$data_y
  preds = out$preds
} 
if(has_models) 
  models = out$models
  
varsExpl_rep = out$varsExpl_rep
cors_rep = out$cors_rep
varsExpl_bg = out$varsExpl_bg
cors_bg = out$cors_bg
argstr = out$argstr
drug_names = colnames(out$cors_rep)
drug_targets = out$drug_targets
dataX = out$dataX
dataY = out$dataY
match_CLs_ext = out$match_CLs_ext

for(fname in input[-1]) {
  env = LoadToEnvironment(fname)
  out = env$out
  
  if(out$argstr != argstr | 
     any(out$drug_targets != drug_targets) |
     any(colnames(out$cors_rep) != drug_names)) {
    print(paste("Incompatible input: ", fname))
    return()
  } else {
    varsExpl_rep = rbind(varsExpl_rep, out$varsExpl_rep)
    cors_rep = rbind(cors_rep, out$cors_rep)
    varsExpl_bg = rbind(varsExpl_bg, out$varsExpl_bg)
    cors_bg = rbind(cors_bg, out$cors_bg)
    if(has_preds) {
      preds_array = abind(preds_array, out$preds_array)
    }
  }
}

cors = apply(cors_rep, 2, mean)
varsExpl = apply(varsExpl_rep, 2, mean)

out = list()
out$cors = cors
out$varsExpl = varsExpl
out$varsExpl_rep = varsExpl_rep
out$cors_rep = cors_rep
out$varsExpl_bg = varsExpl_bg
out$varsExpl_bg_p05 = apply(varsExpl_bg,2,function(x) as.numeric(quantile(x,p=0.95)) )
out$cors_bg = cors_bg
out$cors_bg_p05 = apply(cors_bg,2,function(x) as.numeric(quantile(x,p=0.95)) )
out$argstr = argstr
out$drug_targets = drug_targets
out$dataY = dataY
out$dataX = dataX
out$match_CLs_ext = match_CLs_ext
if(has_preds) {
  out$preds_array = preds_array
  out$data_y = data_y
  out$preds = preds
}
if(has_models)
  out$models = models


pvals_cors = c()
pvals_varsExpl = c()
for(d in seq(1,length(drug_names))) {
  pval_cor = min(1, (length(which(cors_bg[,d]>cors[d]))+1)/length(cors_bg[,d]) )
  pval_varExpl = min(1, (length(which(varsExpl_bg[,d]>varsExpl[d]))+1)/length(varsExpl_bg[,d]) )
  pvals_cors = c(pvals_cors,pval_cor)
  pvals_varsExpl = c(pvals_varsExpl,pval_varExpl)
}
names(pvals_cors) = drug_names
names(pvals_varsExpl) = drug_names
pvals_cors_adj = p.adjust(pvals_cors,method="BH")
pvals_varsExpl_adj = p.adjust(pvals_varsExpl,method="BH")

out$pvals_cors = pvals_cors
out$pvals_cors_adj = pvals_cors_adj
out$pvals_varsExpl = pvals_varsExpl
out$pvals_varsExpl_adj = pvals_varsExpl_adj

m = str_match(argstr,"(.*_)Nbg(.*)$")
argstr = paste(m[2], "Nbg",nrow(cors_rep),sep="")
out$argstr = argstr

save(out, file=output)
quit(status=0)

a=1
b=2
d=a+b





